"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

exports.default = function () {

    return initConfig().then(function () {
        return showMenu();
    }).then(function (res) {
        switch (res) {
            case "globals":
                return configGlobals();
            case "createb":
                return addNewBridge();
            case "editb":
                return editBridge();
            case "deleteb":
                return deleteBridges();
            case "exit":
                return;
        }
    }).catch(CancelSignal, function () {
        process.exit(0);
    }).catch(function (err) {
        process.stderr.write(err.message + "\n");
        process.exit(1);
    });
};

var _inquirer = require("inquirer");

var _inquirer2 = _interopRequireDefault(_inquirer);

var _debug = require("debug");

var _debug2 = _interopRequireDefault(_debug);

var _etc = require("etc");

var _etc2 = _interopRequireDefault(_etc);

var _lodash = require("lodash");

var _config = require("../config");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var Promise = require("bluebird");

// RFC1459
var IRC_NICK_RE = /^[a-z_\-\[\]\\^{}|`][a-z0-9_\-\[\]\\^{}|`]{2,15}$/i;
/*eslint no-control-regex: "off"*/
var IRC_CHAN_RE = /^([#&][^\x07\x2C\s]{0,200})$/i;

var debug = (0, _debug2.default)("cli-config");

var CancelSignal = function CancelSignal() {};

var options = [{
    key: "globals",
    text: "(re)Configure global settings"
}, {
    key: "editb",
    text: "Edit bridge config"
}, {
    key: "createb",
    text: "Create new bridge config"
}, {
    key: "deleteb",
    text: "Delete bridge(s)"
}, {
    key: "exit",
    text: "Exit"
}];

var generateConfigQuestions = function generateConfigQuestions() {
    var defaults = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


    var defaultsOptions = (0, _lodash.assignIn)({}, {
        "user": _config.config.get("user") || undefined,
        "group": _config.config.get("group") || undefined,
        "telegram:token": _config.config.get("telegram:token") || undefined,
        "irc:server": _config.config.get("irc:server") || undefined,
        "irc:port": _config.config.get("irc:port") || 6697,
        "irc:nick": _config.config.get("irc:nick") || undefined,
        "irc:secure": _config.config.get("irc:secure") === false ? false : _config.config.get("irc:secure"),
        "oneConnectionByUser": _config.config.get("oneConnectionByUser") || false,
        "ircScapeCharacter": _config.config.get("ircScapeCharacter") || "",
        "showJoinLeft": _config.config.get("showJoinLeft") || true,
        "prefix": _config.config.get("prefix") || "telegram_",
        "suffix": _config.config.get("suffix") || ""
    }, defaults);

    var suQuestions = [];

    if (process.getuid() === 0 && !options.ignoreSuQuestions) {
        suQuestions = [{
            type: "input",
            name: "user",
            message: "Daemon UID",
            default: defaultsOptions["user"]
        }, {
            type: "input",
            name: "group",
            message: "Daemon GID",
            default: defaultsOptions["group"]
        }];
    }

    return [].concat(_toConsumableArray(suQuestions), [{
        type: "input",
        name: "telegram:token",
        message: "Telegram token",
        default: defaultsOptions["telegram:token"]
    }, {
        type: "input",
        name: "irc:server",
        message: "IRC server ip or nameserver?",
        default: defaultsOptions["irc:server"]
    }, {
        type: "input",
        name: "irc:port",
        message: "IRC server port?",
        default: defaultsOptions["irc:port"],
        validate: function validate(value) {
            value = parseInt(value);
            return value > 0 && value < 65535;
        }
    }, {
        type: "input",
        name: "irc:nick",
        message: "IRC nickname",
        default: defaultsOptions["irc:nick"],
        validate: function validate(value) {

            if (!options.ircNickNameRequired && value === "") {
                return true;
            }

            if (value.match(IRC_NICK_RE)) {
                return true;
            } else {
                return "Invalid IRC nick";
            }
        }
    }, {
        type: "confirm",
        name: "irc:secure",
        message: "IRC server uses SSL?",
        default: defaultsOptions["irc:secure"]
    }, {
        type: "confirm",
        name: "oneConnectionByUser",
        message: "Do you use one new IRC connection by telgram user?",
        default: defaultsOptions["oneConnectionByUser"]
    }, {
        type: "input",
        name: "ircScapeCharacter",
        message: "Do you use one especial character for IRC bots?",
        default: defaultsOptions["ircScapeCharacter"]
    }, {
        type: "confirm",
        name: "showJoinLeft",
        message: "Do you want to show who join or left irc channel in Telegram?",
        default: defaultsOptions["showJoinLeft"]
    }, {
        type: "input",
        name: "prefix",
        message: "Prefix for IRC users?",
        default: defaultsOptions["prefix"],
        validate: function validate(value) {
            return !!value.match(/^[a-z0-9_]{0,10}$/i);
        }
    }, {
        type: "input",
        name: "suffix",
        message: "Suffix for IRC users?",
        default: defaultsOptions["suffix"],
        validate: function validate(value) {
            return !!value.match(/^[a-z0-9_]{0,10}$/i);
        }
    }]);
};

var generateBridgeConfigQuestions = function generateBridgeConfigQuestions() {
    var defaults = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};


    var defaultsOptions = (0, _lodash.assignIn)({}, {
        "enable": true
    }, defaults);

    return [{
        type: "input",
        name: "name",
        message: "Bridge name",
        default: defaultsOptions["name"],
        validate: function validate(value) {
            var bridges = _config.config.get("bridges").map(function (x) {
                return x.name;
            });

            if (bridges.indexOf(value) !== -1 && value !== defaults.name) {
                return "Bridge already exists";
            }

            if (value.match(/^[a-z0-9]{3,20}$/i)) {
                return true;
            } else {
                return "Bridge name can be letters, numbers and _ " + "character, and a length from 3 to 20";
            }
        }
    }].concat(_toConsumableArray(generateConfigQuestions(defaultsOptions, {
        ircNickNameRequired: true,
        ignoreSuQuestions: true
    })), [{
        type: "input",
        name: "irc:channel",
        message: "IRC channel",
        default: defaultsOptions["irc:channel"],
        validate: function validate(val) {
            if (val.match(IRC_CHAN_RE)) {
                return true;
            } else {
                return "Invalid IRC channel name";
            }
        }
    }, {
        type: "input",
        name: "telegram:channel",
        message: "Telegram channel name or id",
        default: defaultsOptions["telegram:channel"],
        validate: function validate(val) {
            if (!val || val === "") {
                return "You need to define a Telegram channel";
            }

            return true;
        }
    }, {
        type: "confirm",
        name: "enable",
        message: "Bridge is enable?",
        default: defaultsOptions["enable"]
    }]);
};

var confirm = function confirm(text) {

    return _inquirer2.default.prompt({
        type: "confirm",
        name: "res",
        message: text
    }).then(function (res) {
        return res.res;
    });
};

var initConfig = function initConfig() {

    return new Promise(function (resolve) {
        return resolve((0, _config.checkConfigDir)());
    }).then(function (configExists) {
        if (!configExists) {
            return confirm("Config directory does not exists, you can to create it?");
        } else {
            return false;
        }
    }).then(function (doesCreateConfigDir) {
        if (doesCreateConfigDir) {
            return (0, _config.createConfigDir)();
        } else {
            return false;
        }
    }).then(function () {
        return (0, _config.checkDataDir)();
    }).then(function (dataDirExists) {
        if (!dataDirExists) {
            return confirm("Data directory does not exists, you can to create it?");
        } else {
            return false;
        }
    }).then(function (doesCreateDataDir) {
        if (doesCreateDataDir) {
            return (0, _config.createDataDir)();
        } else {
            return false;
        }
    }).then(function () {
        if (!(0, _config.checkConfigDir)() || !(0, _config.checkDataDir)()) {
            throw new CancelSignal();
        }
    });
};

var showMenu = function showMenu() {

    return _inquirer2.default.prompt({
        type: "list",
        name: "menu",
        message: "What do you want?",
        choices: options.map(function (o) {
            return o.text;
        })
    }).then(function (res) {
        return options.find(function (o) {
            return o.text === res.menu;
        }).key;
    });
};

var configGlobals = function configGlobals() {

    return _inquirer2.default.prompt(generateConfigQuestions()).then(function (gconfig) {
        for (var i in gconfig) {
            var v = gconfig[i];
            if (v === "") {
                continue;
            }
            _config.config.set(i, gconfig[i]);
        }

        debug("RR", _config.config.toJSON());

        var configResult = (0, _config.renderConfigFile)();

        process.stdout.write(configResult);
        process.stdout.write("\n\n");

        return confirm("Confirm save?");
    }).then(function (save) {
        if (save) {
            return (0, _config.saveConfig)();
        } else {
            throw new CancelSignal();
        }
    });
};

var addNewBridge = function addNewBridge() {
    var bconfig = (0, _etc2.default)();

    var questions = generateBridgeConfigQuestions();

    return _inquirer2.default.prompt(questions).then(function (gconfig) {
        for (var i in gconfig) {
            var v = gconfig[i];
            if (v === "") {
                continue;
            }
            bconfig.set(i, v);
        }

        debug("RR", bconfig.toJSON());

        var configResult = (0, _config.renderConfigFile)(bconfig.toJSON());

        process.stdout.write(configResult);
        process.stdout.write("\n\n");

        return confirm("Confirm save?");
    }).then(function (save) {
        if (save) {
            return (0, _config.saveBridgeConfig)(bconfig.toJSON());
        } else {
            throw new CancelSignal();
        }
    });
};

var editBridge = function editBridge() {
    var bridges = _config.config.get("bridges").map(function (x) {
        return x.name;
    });
    var bconfig = (0, _etc2.default)();
    var originalName = void 0;

    debug("Edit bridge menu", bridges);

    if (!bridges.length) {
        return Promise.reject(new Error('No bridges to edit'));
    }

    return _inquirer2.default.prompt({
        type: "list",
        name: "bridge",
        choices: bridges,
        message: "Select bridge to edit"
    }).then(function (res) {
        debug("select", res);

        var bridgeConfig = (0, _config.getBridgeConfig)(res.bridge);
        originalName = res.bridge;
        bconfig.add(bridgeConfig);

        var defaultsOptions = {
            "name": bconfig.get("name"),
            "enable": bconfig.get("enable"),
            "telegram:token": bconfig.get("telegram:token"),
            "telegram:channel": bconfig.get("telegram:channel"),
            "irc:server": bconfig.get("irc:server"),
            "irc:port": bconfig.get("irc:port"),
            "irc:nick": bconfig.get("irc:nick"),
            "irc:channel": bconfig.get("irc:channel"),
            "irc:secure": bconfig.get("irc:secure") === false ? false : bconfig.get("irc:secure"),
            "oneConnectionByUser": bconfig.get("oneConnectionByUser"),
            "ircScapeCharacter": bconfig.get("ircScapeCharacter"),
            "showJoinLeft": bconfig.get("showJoinLeft"),
            "prefix": bconfig.get("prefix"),
            "suffix": bconfig.get("suffix")
        };

        return _inquirer2.default.prompt(generateBridgeConfigQuestions(defaultsOptions));
    }).then(function (gconfig) {
        for (var i in gconfig) {
            var v = gconfig[i];
            if (v === "") {
                continue;
            }
            bconfig.set(i, v);
        }

        debug("RR", bconfig.toJSON());

        var configResult = (0, _config.renderConfigFile)(bconfig.toJSON());

        process.stdout.write(configResult);
        process.stdout.write("\n\n");

        return confirm("Confirm save?");
    }).then(function (save) {
        if (save) {
            (0, _config.deleteBridgeConfig)(originalName);
            return (0, _config.saveBridgeConfig)(bconfig.toJSON());
        } else {
            throw new CancelSignal();
        }
    });
};

var deleteBridges = function deleteBridges() {
    var bridges = _config.config.get("bridges").map(function (x) {
        return x.name;
    });

    debug("Delete bridges menu");

    if (!bridges.length) {
        debug("No bridges to delete");
        return Promise.reject(new Error('No bridges to delete'));
    }

    return _inquirer2.default.prompt({
        type: "checkbox",
        name: "bridges",
        choices: bridges,
        message: "Select bridges to delete",
        validate: function validate(val) {
            return !!val.length;
        }
    }).then(function (res) {
        return _inquirer2.default.prompt({
            type: "confirm",
            name: "deleteall",
            default: false,
            message: "You are going to delete the next bridges " + "configurations: " + res.bridges.join(", ") + ". Are you sure?"
        }).then(function (cres) {
            if (cres.deleteall) {
                res.bridges.map(function (b) {
                    return (0, _config.deleteBridgeConfig)(b);
                });
            } else {
                throw new CancelSignal();
            }
        });
    });
};